@@ -3,7 +3,7 @@
 use std::{net::IpAddr, time::Duration};
 
 /// Options for connecting to TCP remote server
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Default)]
 pub struct TcpSocketOpts {
     /// TCP socket's `SO_SNDBUF`
     pub send_buffer_size: Option<u32>,
@@ -22,20 +22,8 @@ pub struct TcpSocketOpts {
     pub keepalive: Option<Duration>,
 }
 
-impl Default for TcpSocketOpts {
-    fn default() -> TcpSocketOpts {
-        TcpSocketOpts {
-            send_buffer_size: None,
-            recv_buffer_size: None,
-            nodelay: false,
-            fastopen: false,
-            keepalive: None,
-        }
-    }
-}
-
 /// Options for connecting to remote server
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Default)]
 pub struct ConnectOpts {
     /// Linux mark based routing, going to set by `setsockopt` with `SO_MARK` option
     #[cfg(any(target_os = "linux", target_os = "android"))]
@@ -53,39 +41,15 @@ pub struct ConnectOpts {
     pub bind_local_addr: Option<IpAddr>,
 
     /// Outbound socket binds to interface
-    #[cfg(any(target_os = "linux", target_os = "android", target_os = "macos", target_os = "ios"))]
     pub bind_interface: Option<String>,
 
     /// TCP options
     pub tcp: TcpSocketOpts,
 }
 
-impl Default for ConnectOpts {
-    fn default() -> ConnectOpts {
-        ConnectOpts {
-            #[cfg(any(target_os = "linux", target_os = "android"))]
-            fwmark: None,
-            #[cfg(target_os = "android")]
-            vpn_protect_path: None,
-            bind_local_addr: None,
-            #[cfg(any(target_os = "linux", target_os = "android", target_os = "macos", target_os = "ios"))]
-            bind_interface: None,
-            tcp: TcpSocketOpts::default(),
-        }
-    }
-}
-
 /// Inbound connection options
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, Default)]
 pub struct AcceptOpts {
     /// TCP options
     pub tcp: TcpSocketOpts,
 }
-
-impl Default for AcceptOpts {
-    fn default() -> AcceptOpts {
-        AcceptOpts {
-            tcp: TcpSocketOpts::default(),
-        }
-    }
-}
